import { makeStyles } from "@material-ui/core/styles";
import { styled } from '@mui/material/styles';
import { useState,useRef } from "react";
import { IconButton } from "@material-ui/core";
import InsertDriveFileIcon from '@mui/icons-material/InsertDriveFile';
import FileDownloadIcon from '@mui/icons-material/FileDownload';
import Button from '@mui/material/Button';
import ContentCopyIcon from '@mui/icons-material/ContentCopy';
import DeleteOutlineIcon from '@mui/icons-material/DeleteOutline';
import Menu from '@mui/material/Menu';
import MenuItem from '@mui/material/MenuItem';
import Divider from '@mui/material/Divider';
import CheckBoxIcon from '@mui/icons-material/CheckBox';
import Checkbox from '@mui/material/Checkbox';
import PushPinIcon from '@mui/icons-material/PushPin';
import CloseIcon from '@mui/icons-material/Close';
import ZoomOutIcon from '@mui/icons-material/ZoomOut';
import ZoomInIcon from '@mui/icons-material/ZoomIn';
import ReplyIcon from '@mui/icons-material/Reply';
import { CopyToClipboard } from 'react-copy-to-clipboard';
import { removeMessageById,updateMessageById,pinMessageById } from "../../../../../../api-data";
import { timeStampMessage,copied,emojisArr } from '../../../../../../helpers'

const FileViewer =  require('react-file-viewer')

const StyledMenu = styled((props:any) => (
  <Menu
    elevation={0}
    anchorOrigin={{
      vertical: 'top',
      horizontal: 'right',
    }}
    transformOrigin={{
      vertical: 'bottom',
      horizontal: 'right',
    }}
    {...props}
  />
))(({ theme }:any) => ({
  '& .MuiPaper-root': {
    borderRadius: 10,
    marginTop: theme.spacing(0),
    minWidth: 220,
    color:
      theme.palette.mode === 'light' ? 'rgb(55, 65, 81)' : theme.palette.grey[500],
    boxShadow:
      'rgb(255, 255, 255) 0px 0px 0px 0px, rgba(0, 0, 0, 0.05) 0px 0px 0px 1px, rgba(0, 0, 0, 0.1) 0px 10px 15px -3px, rgba(0, 0, 0, 0.05) 0px 4px 6px -2px',
    '& .MuiMenu-list': {
       padding: '4px 4px',
    },
    '& .MuiMenuItem-root': {
      marginBottom: theme.spacing(1),
      '& .MuiSvgIcon-root': {
        fontSize: 21,
        color: theme.palette.text.secondary,
        marginRight: theme.spacing(2),
      }
    },
  },
}));

const useStyles = makeStyles({
  container: {
    display: "flex",
    justifyContent: "flex-start",
    borderRadius: 7,
  },
  wrapper: {
    position: 'relative',
    display: 'flex',
    justifyContent: 'space-between',
    alignContent: 'center',
    alignItems: 'center',
    width: 200,
    padding: '5px 5px 22px 5px',
    backgroundColor: '#ffffff',
    borderRadius: 7,
      "&:after": {
        content: "''",
        position: "absolute",
        width: "0",
        height: "0",
        borderBottom: "15px solid #ffffff",
        borderLeft: "15px solid transparent",
        borderRight: "15px solid transparent",
        bottom: '0px',
        left: "-15px"
      },
      "&:before": {
        content: "''",
        position: "absolute",
        width: "0",
        height: "0",
        borderBottom: "17px solid #ffffff",
        borderLeft: "16px solid transparent",
        borderRight: "16px solid transparent",
        bottom: "0px",
        left: "-17px"
      }     
  },
  wrapperActive: {
    position: 'relative',
    display: 'flex',
    justifyContent: 'space-between',
    alignContent: 'center',
    alignItems: 'center',
    width: 200,
    padding: '5px 5px 22px 5px',
    backgroundColor: '#babdbc',
    borderRadius: 7,
      "&:after": {
        content: "''",
        position: "absolute",
        width: "0",
        height: "0",
        borderBottom: "15px solid #babdbc",
        borderLeft: "15px solid transparent",
        borderRight: "15px solid transparent",
        bottom: '0px',
        left: "-15px"
      },
      "&:before": {
        content: "''",
        position: "absolute",
        width: "0",
        height: "0",
        borderBottom: "17px solid #babdbc",
        borderLeft: "16px solid transparent",
        borderRight: "16px solid transparent",
        bottom: "0px",
        left: "-17px"
      }     
  },  
  bntDownload: {
    backgroundColor: 'inherit',
    color: '#54b0fc',
    width: 30,
    height:30,
    '&:hover': {
      backgroundColor: '#54b0fc',
      color:'#ffffff'
    }
  },
  title: {
    margin: '0 30px 0 5px',
    color: '#0e0d0d',
    cursor: 'pointer',
    '&:hover': {
      color: '#54b0fc',
    }
  },
  time: {
      position: "absolute",
      fontSize: ".65em",
      fontWeight:600,
      bottom: 0,
      right: 6,
      color: '#414141',
      padding: 3,
      borderRadius: 5,
  },
  captionWrapper: {
      position: "absolute",
      fontSize: ".85em",
      color: '#414141',
      maxWidth:'76%',
      fontWeight:600,
      bottom: -28,
      left: 12,
      borderRadius: 5,
      wordBreak:'break-word',
      textAlign: "left",
      font: "400 .9em 'Open Sans', sans-serif",
      backgroundColor: '#deffa9',
      padding:10,
      "&:after": {
        content: "''",
        position: "absolute",
        width: "0",
        height: "0",
        borderBottom: "15px solid #deffa9",
        borderLeft: "15px solid transparent",
        borderRight: "15px solid transparent",
        bottom: '0px',
        left: "-15px"
      },
      "&:before": {
        content: "''",
        position: "absolute",
        width: "0",
        height: "0",
        borderBottom: "17px solid #deffa9",
        borderLeft: "16px solid transparent",
        borderRight: "16px solid transparent",
        bottom: "-1px",
        left: "-17px"
      }      
  }, 
  overlay: {
    position: 'absolute',
    top: 0,
    left: 0,
    width: '100%',
    height:'100%',
    backgroundColor: 'rgba(104, 105, 104, 0.6)',
    border:'solid 2px #0084ff',
    cursor: 'auto',
    zIndex: 100,
    overflow: 'auto',
  },
  wrapperIcons: {
    position: 'absolute',
    top: 14,
    right: 40,
    display: 'flex',
    alignContent: 'center',
    alignItems: 'center',
    padding: '2px 10px',
    borderRadius: 10,
    backgroundColor: 'rgba(65, 65, 65, 0.9)',
    zIndex:150
  },  
  magnifying : {
    cursor: 'pointer',
    color: '#e9e7e7',
    padding: 0,
    marginRight:10,
    '&:hover': {
      color: '#ffffff',
      transform: 'scale(1.1)'
    }
  },
  iconCloseOverlay: {
    marginLeft:5,
    cursor: 'pointer',
    color: '#e9e7e7',
    padding: 0,
    '&:hover': {
      color: '#ffffff',
      transform: 'rotate(180deg)',
      transition: 'all 250ms ease-out ',
    }
  },  
  modalDelete: {
    background: '#ffffff',
    position: 'absolute',
    content:'',
    width: '20%',
    height:'auto',
    left: '40%',
    bottom: '48.5%',
    borderRadius: 10,
    padding: 10,
    display: 'flex',
    flexDirection:'column'
  },  
  overlayDelete: {
    position: 'fixed',
    top: 0,
    left: 0,
    width: '100vw',
    height: '100vh',
    zIndex: 100,
    backgroundColor: 'rgba(104, 105, 104, 0.6)',
    overflowY: 'hidden',
  },
  emojiTitle: {
      position: "absolute",
      fontSize: "1.7em",
      fontWeight:600,
      bottom: 0,
      right: -40,
  },
  emojiCompanionTitle: {
      position: "absolute",
      fontSize: "1.7em",
      fontWeight:600,
      bottom: '2rem',
      right: -40,
  },  
  emoji: {
		cursor: 'pointer',
		fontSize: '1.7rem',
    transition: 'all 0.3s',
    '&:hover': {
      transform: 'scale(1.5)'
    }
  },
  emojiActive: {
    cursor: 'pointer',
    fontSize: '1.2rem',
    animation: `$emoji 0.6s ease-out`,
		animationDirection: 'forwards',
		animationIterationCount: 1,
  },  
  '@keyframes emoji': {
	  '5%': { transform: 'translateY(1rem)'},
	  '10%': { transform: 'translateY(0) scale(1)',opacity: 1},
	  '50%': { transform: 'translateY(-4rem) scale(1.5) rotateY(90deg)'},
	  '80%': {opacity: 0},
	  '100%': {transform: 'translateY(-8rem) scale(2) rotateY(180deg)',opacity: 0},
  },
  iconClose: {
    '&:hover': {
      transform: 'rotate(180deg)',
      transition: 'all 250ms ease-out ',
    }
  },  
});

const label = { inputProps: { 'aria-label': 'Checkbox demo' } };

interface IMessageLeftFile {
  url:string,
  createdAt: string,
  type: string,
  caption: string,
  emoji: string,
  emojiCompanion: string,
  pinned: boolean,
  isSomeSelected: boolean,
  isSelected:(_id:string) => boolean,
  handleSelected: (_id:string) => void,  
  _id: string,
  nightMode: boolean,
  handleReply: (_id: string) => void
}

const MessageLeftFile = ({ url,createdAt,type,caption,emoji,emojiCompanion,pinned,isSomeSelected,isSelected,handleSelected,_id,nightMode,handleReply }:IMessageLeftFile) => {
  const classes = useStyles();
  const [read, setRead] = useState<boolean>(false)
  const [anchorEl, setAnchorEl] = useState<any>(null);
  const [selected, setSelected] = useState<boolean>(false);
  const [scale, setScale] = useState<number>(1)
  const [modal, setModal] = useState<boolean>(false)
  const refView = useRef<null | any>(null)
  const open = Boolean(anchorEl);
  const checked = isSelected(_id)
  const handleIncreaseScale = () => {
    if (scale < 5) {
      refView.current.style.transform = `scale(${scale+0.25},${scale+0.25})`
      refView.current.style.transformOrigin = `${50 / scale + 0.25}px ${50 / scale + 0.25}px`
      setScale(scale+0.25)
    }
  }
  const handleDecreaseScale = () => {
    if (scale >= 0.5) {
      refView.current.style.transform = `scale(${scale-0.25},${scale-0.25})`
      refView.current.style.transformOrigin = `${50/scale-0.25}px ${50/scale-0.25}px`
      setScale(scale-0.25)
    }
  }   
  const handleOpenRead = () => !read&&setRead(true)
  const handleCloseRead = (e:any) => {
    const id = e.target.id
    if (id === 'overlay' || id === 'close') {
      setRead(false)
      setScale(1)
    }
  }
  const handleClose = (type: string | undefined): void => {
    if (type === 'copy') copied('Link')
    if (type === 'delete') setModal(true)
    setAnchorEl(null)
    setSelected(false)
  }

  const handleDeleteModal = (e: any) => {
    const id = e.target.id
    if (id === 'overlay' || id === 'cancel') return setModal(false)
    if (id === 'delete') {
      removeMessageById(_id)
      setModal(false)
    }
  }  
  const handleContextMenu = (e: React.MouseEvent<HTMLDivElement>):void => {
    e.preventDefault()
    setAnchorEl(e.currentTarget)
    setSelected(true)
  }
 
  const handleEmojiMenu = ({ target }: any): void => {
    const idEmoji = target.id
    if (idEmoji === emoji) {updateMessageById(_id,'')  
    } else updateMessageById(_id,idEmoji)
  }   

  return (
    <div className={classes.container} style={{ marginBottom: caption ? 43 : 15}}>
      {isSomeSelected&&<Checkbox {...label} checked={checked} sx={{color:nightMode?'#ffffff':'#00ff48','&.Mui-checked': {color: nightMode?'#ffffff':'#00ff48'}}} onClick={() => handleSelected(_id)}/>}
      {read && <div className={classes.wrapperIcons}>
        <ZoomOutIcon onClick={handleDecreaseScale} className={classes.magnifying} fontSize='large' />
        <ZoomInIcon onClick={handleIncreaseScale} className={classes.magnifying} fontSize='large' />
        <CloseIcon id='close' onClick={handleCloseRead} className={classes.iconCloseOverlay} fontSize='large' />
      </div>}        
      {read && <div className={classes.overlay} id='overlay'
        onClick={handleCloseRead}>
        <div ref={refView}>
          <FileViewer
            allowFullScreen={true}
            fileType={type}
            filePath={url}
            onError={handleCloseRead}
          />
        </div>
      </div>}
      <div onContextMenu={(e) => handleContextMenu(e)}
        className={selected ? classes.wrapperActive : classes.wrapper}
        style={{pointerEvents:isSomeSelected?'none':'auto'}}>
        <InsertDriveFileIcon fontSize='large' style={{ color: '#99b401' }} />
        {!read && <span className={classes.title} onClick={handleOpenRead}>Read File</span>}
        <a href={url} target="_blank" rel="noreferrer" download>
           <IconButton className={classes.bntDownload}  >
             <FileDownloadIcon fontSize='medium'/>
           </IconButton>
        </a>
        <div className={classes.time}>{timeStampMessage(createdAt)}</div>
        {emojiCompanion && <div className={classes.emojiCompanionTitle}>{emojisArr[Number(emojiCompanion)]}</div>}
        {emoji && <div className={classes.emojiTitle}>{emojisArr[Number(emoji)]}</div>}
        {caption&&<div className={classes.captionWrapper}>{caption}</div>}
      <StyledMenu id="demo-positioned-menu" aria-labelledby="demo-positioned-button"
          anchorEl={anchorEl} open={open} onClose={handleClose}>
          <MenuItem onClick={handleEmojiMenu} style={{ cursor: 'none' }} >
            {emojisArr.map((el:string, i:number) =>
              <div key={el} className={emoji === String(i)?classes.emojiActive:classes.emoji} id={String(i)}>{el}</div>)}
          </MenuItem>
        <Divider />
        <MenuItem onClick={() => {
          handleReply(_id)
          handleClose(undefined)
        }}>
          <ReplyIcon />
          Reply
        </MenuItem>
        <MenuItem>
            <ReplyIcon style={{transform :'rotateY(180deg)'}} />
            Forward
        </MenuItem>           
        <CopyToClipboard onCopy={() => handleClose('copy')} text={url}>
          <MenuItem>
            <ContentCopyIcon />
             Copy Link
          </MenuItem>
        </CopyToClipboard>
        <MenuItem onClick={() => {
          pinMessageById(_id, !pinned)
          handleClose(undefined)
        }}>
            {pinned ?
              <CloseIcon className={classes.iconClose} /> :
              <PushPinIcon />}
             {pinned?'Unpin':'Pin'}
        </MenuItem>       
        <MenuItem onClick={() => {
            handleSelected(_id)
            handleClose(undefined)
          }}>
            <CheckBoxIcon />
            Select
        </MenuItem>
        <MenuItem style={{color:'#f02a2a'}} onClick={() => handleClose('delete')}>
            <DeleteOutlineIcon style={{color:'#f02a2a'}}/>
            Delete
        </MenuItem>        
      </StyledMenu>
      {modal &&
      <div onClick={handleDeleteModal} className={classes.overlayDelete} id='overlay'>
        <div className={classes.modalDelete}>
          <h3 style={{color: '#2c2c2c'}}>Delete message</h3>
          <p style={{ color: '#050505' }}>Are you sure you want to delete message?</p>
          <Button id='delete' variant="text" color="error" style={{fontWeight:500,fontSize:22}}>
            DELETE MESSAGE
          </Button>         
          <Button id='cancel' variant="text" style={{fontWeight:500,fontSize:22}}>
             CANCEL
          </Button>
        </div>  
      </div>}         
      </div>
    </div>        
)};

export  default  MessageLeftFile
